Module allocator

Module allocator 

Memory arena allocator used by all other submodules.

See the oxc_allocator module-level documentation for more information.

Modules

hash_map
A hash map without Drop that stores data in arena allocator.
hash_set
A hash set without Drop that stores data in arena allocator.

Structs

Address
Memory address of an AST node in arena.
Allocator
A bump-allocated memory arena.
ArenaBox
A Box without Drop, which stores its data in the arena allocator.
ArenaHashMap
A hash map without Drop that stores data in arena allocator.
ArenaHashSet
A hash set without Drop that stores data in arena allocator.
ArenaStringBuilder
String builder.
ArenaVec
A Vec without Drop, which stores its data in the arena allocator.
BitSet
A bitset allocated in an arena.
Box
A Box without Drop, which stores its data in the arena allocator.
HashMap
A hash map without Drop that stores data in arena allocator.
HashSet
A hash set without Drop that stores data in arena allocator.
StringBuilder
String builder.
Vec
A Vec without Drop, which stores its data in the arena allocator.

Enums

CloneInSemanticIds
Option to pass to CloneIn::clone_in_impl to determine how to clone semantic IDs.

Traits

CloneIn
A trait to explicitly clone an object into an arena allocator.
Dummy
A trait to create a dummy AST node.
FromIn
This trait works similarly to the standard library From trait.
GetAddress
Trait for getting the memory address of an AST node.
GetAllocator
Accessor for getting the underlying allocator.
IntoIn
This trait works similarly to the standard library Into trait. It is similar to FromIn is reflective, A FromIn implementation also implicitly implements IntoIn for the opposite type.
ReplaceWith
A trait to replace an existing AST node in place with a new node built from the old one.
TakeIn
A trait to replace an existing AST node with a dummy.
UnstableAddress
Trait for getting the memory address of an AST node which is not necessarily stable.