Box

Struct Box 

pub struct Box<'alloc, T>(/* private fields */)
where
    T: ?Sized;
A Box without Drop, which stores its data in the arena allocator.

No Drops

Objects allocated into Oxc memory arenas are never Dropped. Memory is released in bulk when the allocator is dropped, without dropping the individual objects in the arena.

Therefore, it would produce a memory leak if you allocated Drop types into the arena which own memory allocations outside the arena.

Static checks make this impossible to do. Box::new_in will refuse to compile if called with a Drop type.

Implementations

Source

impl<'alloc, T> Box<'alloc, T>

pub fn new_in(value: T, allocator: &impl GetAllocator<'alloc>) -> Box<'alloc, T>

Allocate value into the memory arena, and receive a Box which owns the value.

Examples
use oxc_allocator::{Allocator, Box};

let arena = Allocator::default();
let arena = &arena;
let in_arena: Box<i32> = Box::new_in(5, &arena);

The Box cannot outlive the Allocator. This fails to compile:

use oxc_allocator::{Allocator, Box};

let boxed = {
    let allocator = Allocator::default();
    let allocator = &allocator;
    Box::new_in(5, &allocator)
};
assert_eq!(*boxed, 5);
pub fn unbox(self) -> T

Take ownership of the value stored in this Box, consuming the box in the process.

Examples
use oxc_allocator::{Allocator, Box};

let arena = Allocator::default();
let arena = &arena;

// Put `5` into the arena and on the heap.
let boxed: Box<i32> = Box::new_in(5, &arena);
// Move it back to the stack. `boxed` has been consumed.
let i = boxed.unbox();

assert_eq!(i, 5);
Source

impl<T> Box<'_, T>
where T: ?Sized,

pub fn as_non_null(boxed: &Box<'_, T>) -> NonNull<T>

Get a NonNull pointer pointing to the Box’s contents.

The pointer is not valid for writes.

The caller must ensure that the Box outlives the pointer this function returns, or else it will end up dangling.

Example
use oxc_allocator::{Allocator, Box};

let allocator = Allocator::new();
let allocator = &allocator;
let boxed = Box::new_in(123_u64, &allocator);
let ptr = Box::as_non_null(&boxed);
pub fn into_non_null(boxed: Box<'_, T>) -> NonNull<T>

Consume a Box and return a NonNull pointer to its contents.

pub const unsafe fn from_non_null(ptr: NonNull<T>) -> Box<'_, T>

Create a Box from a NonNull pointer.

SAFETY
  • Pointer must point to a valid T.
  • Pointer must point to within an Allocator.
  • Caller must ensure that the pointer is valid for the lifetime of the Box.
Source

impl<T> Box<'static, [T]>

pub fn new_empty_boxed_slice() -> Box<'static, [T]>

Create a new empty Box<[T]>.

This method does not allocate. The returned boxed slice is represented by a dangling, correctly-aligned pointer with length 0, similar to how Vec::new_in produces an empty vector.

Source

impl<'alloc, T> Box<'alloc, [T]>

pub fn into_arena_slice(self) -> &'alloc [T]

Convert a boxed slice [Box<[T]>] into slice [&'alloc [T]].

The returned slice has the same lifetime as the allocator.

pub fn into_arena_slice_mut(self) -> &'alloc mut [T]

Convert a boxed slice [Box<[T]>] into mutable slice [&'alloc mut [T]].

The returned slice has the same lifetime as the allocator.