#[non_exhaustive]#[repr(C)]pub struct BlockStatement<'a> {
pub span: Span,
pub node_id: Cell<NodeId>,
pub scope_id: Cell<Option<ScopeId>>,
pub body: Vec<'a, Statement<'a>>,
}Expand description
{ let foo = 1; } in if(true) { let foo = 1; }
Represents a block statement, which can include a body.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.span: Spannode_id: Cell<NodeId>scope_id: Cell<Option<ScopeId>>body: Vec<'a, Statement<'a>>Implementations
Sourceimpl<'a> BlockStatement<'a>
impl<'a> BlockStatement<'a>
Sourcepub fn new<B, T1>(span: Span, body: T1, builder: &B) -> BlockStatement<'a>
pub fn new<B, T1>(span: Span, body: T1, builder: &B) -> BlockStatement<'a>
Build a BlockStatement.
If you want the built node to be allocated in the memory arena,
use BlockStatement::boxed instead.
Parameters
span: TheSpancovering this nodebody
Sourcepub fn boxed<B, T1>(
span: Span,
body: T1,
builder: &B,
) -> Box<'a, BlockStatement<'a>>
pub fn boxed<B, T1>( span: Span, body: T1, builder: &B, ) -> Box<'a, BlockStatement<'a>>
Build a BlockStatement, and store it in the memory arena.
Returns a Box containing the newly-allocated node.
If you want a stack-allocated node, use BlockStatement::new instead.
Parameters
span: TheSpancovering this nodebody
Sourcepub fn new_with_scope_id<B, T1>(
span: Span,
body: T1,
scope_id: ScopeId,
builder: &B,
) -> BlockStatement<'a>
pub fn new_with_scope_id<B, T1>( span: Span, body: T1, scope_id: ScopeId, builder: &B, ) -> BlockStatement<'a>
Build a BlockStatement with scope_id.
If you want the built node to be allocated in the memory arena,
use BlockStatement::boxed_with_scope_id instead.
Parameters
span: TheSpancovering this nodebodyscope_id
Sourcepub fn boxed_with_scope_id<B, T1>(
span: Span,
body: T1,
scope_id: ScopeId,
builder: &B,
) -> Box<'a, BlockStatement<'a>>
pub fn boxed_with_scope_id<B, T1>( span: Span, body: T1, scope_id: ScopeId, builder: &B, ) -> Box<'a, BlockStatement<'a>>
Build a BlockStatement with scope_id, and store it in the memory arena.
Returns a Box containing the newly-allocated node.
If you want a stack-allocated node, use BlockStatement::new_with_scope_id instead.
Parameters
span: TheSpancovering this nodebodyscope_id
Sourceimpl BlockStatement<'_>
impl BlockStatement<'_>
Sourcepub fn node_id(&self) -> NodeId
pub fn node_id(&self) -> NodeId
Get NodeId of BlockStatement.
Only use this method on a post-semantic AST where NodeIds are always defined.
Sourcepub fn set_node_id(&self, node_id: NodeId)
pub fn set_node_id(&self, node_id: NodeId)
Set NodeId of BlockStatement.
Sourcepub fn scope_id(&self) -> ScopeId
pub fn scope_id(&self) -> ScopeId
Get ScopeId of BlockStatement.
Only use this method on a post-semantic AST where ScopeIds are always defined.
Panics
Panics if scope_id is None.
Sourcepub fn set_scope_id(&self, scope_id: ScopeId)
pub fn set_scope_id(&self, scope_id: ScopeId)
Set ScopeId of BlockStatement.