#[non_exhaustive]#[repr(C)]pub struct ForOfStatement<'a> {
pub span: Span,
pub node_id: Cell<NodeId>,
pub scope_id: Cell<Option<ScopeId>>,
pub left: ForStatementLeft<'a>,
pub right: Expression<'a>,
pub body: Statement<'a>,
pub await: bool,
}Expand description
For-Of Statement
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.span: Spannode_id: Cell<NodeId>scope_id: Cell<Option<ScopeId>>left: ForStatementLeft<'a>right: Expression<'a>body: Statement<'a>await: boolImplementations
Sourceimpl<'a> ForOfStatement<'a>
impl<'a> ForOfStatement<'a>
Sourcepub fn new<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
builder: &B,
) -> ForOfStatement<'a>where
B: GetAstBuilder<'a>,
pub fn new<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
builder: &B,
) -> ForOfStatement<'a>where
B: GetAstBuilder<'a>,
Build a ForOfStatement.
If you want the built node to be allocated in the memory arena,
use ForOfStatement::boxed instead.
Parameters
span: TheSpancovering this nodeawaitleftrightbody
Sourcepub fn boxed<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
builder: &B,
) -> Box<'a, ForOfStatement<'a>>where
B: GetAstBuilder<'a>,
pub fn boxed<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
builder: &B,
) -> Box<'a, ForOfStatement<'a>>where
B: GetAstBuilder<'a>,
Build a ForOfStatement, and store it in the memory arena.
Returns a Box containing the newly-allocated node.
If you want a stack-allocated node, use ForOfStatement::new instead.
Parameters
span: TheSpancovering this nodeawaitleftrightbody
Sourcepub fn new_with_scope_id<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
scope_id: ScopeId,
builder: &B,
) -> ForOfStatement<'a>where
B: GetAstBuilder<'a>,
pub fn new_with_scope_id<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
scope_id: ScopeId,
builder: &B,
) -> ForOfStatement<'a>where
B: GetAstBuilder<'a>,
Build a ForOfStatement with scope_id.
If you want the built node to be allocated in the memory arena,
use ForOfStatement::boxed_with_scope_id instead.
Parameters
span: TheSpancovering this nodeawaitleftrightbodyscope_id
Sourcepub fn boxed_with_scope_id<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
scope_id: ScopeId,
builder: &B,
) -> Box<'a, ForOfStatement<'a>>where
B: GetAstBuilder<'a>,
pub fn boxed_with_scope_id<B>(
span: Span,
await: bool,
left: ForStatementLeft<'a>,
right: Expression<'a>,
body: Statement<'a>,
scope_id: ScopeId,
builder: &B,
) -> Box<'a, ForOfStatement<'a>>where
B: GetAstBuilder<'a>,
Build a ForOfStatement with scope_id, and store it in the memory arena.
Returns a Box containing the newly-allocated node.
If you want a stack-allocated node, use ForOfStatement::new_with_scope_id instead.
Parameters
span: TheSpancovering this nodeawaitleftrightbodyscope_id
Sourceimpl ForOfStatement<'_>
impl ForOfStatement<'_>
Sourcepub fn node_id(&self) -> NodeId
pub fn node_id(&self) -> NodeId
Get NodeId of ForOfStatement.
Only use this method on a post-semantic AST where NodeIds are always defined.
Sourcepub fn set_node_id(&self, node_id: NodeId)
pub fn set_node_id(&self, node_id: NodeId)
Set NodeId of ForOfStatement.
Sourcepub fn scope_id(&self) -> ScopeId
pub fn scope_id(&self) -> ScopeId
Get ScopeId of ForOfStatement.
Only use this method on a post-semantic AST where ScopeIds are always defined.
Panics
Panics if scope_id is None.
Sourcepub fn set_scope_id(&self, scope_id: ScopeId)
pub fn set_scope_id(&self, scope_id: ScopeId)
Set ScopeId of ForOfStatement.