pub struct ElementId(/* private fields */);Implementations
Sourceimpl ElementId
impl ElementId
Sourcepub const MAX_INDEX: usize
pub const MAX_INDEX: usize
If Self::CHECKS_MAX_INDEX is true, we’ll assert if trying to
produce a value larger than this in any of the ctors that don’t
have unchecked in their name.
Sourcepub const CHECKS_MAX_INDEX: bool
pub const CHECKS_MAX_INDEX: bool
Does this index type assert if asked to construct an index larger than MAX_INDEX?
Sourcepub const fn new(value: usize) -> ElementId
pub const fn new(value: usize) -> ElementId
Construct this index type from a usize. Alias for from_usize.
Sourcepub const fn from_raw(value: u32) -> ElementId
pub const fn from_raw(value: u32) -> ElementId
Construct this index type from the wrapped integer type.
Sourcepub fn from_foreign<F>(value: F) -> ElementIdwhere
F: Idx,
pub fn from_foreign<F>(value: F) -> ElementIdwhere
F: Idx,
Construct this index type from one in a different domain
Sourcepub const fn from_usize_unchecked(value: usize) -> ElementId
pub const fn from_usize_unchecked(value: usize) -> ElementId
Construct from a usize without any checks.
Sourcepub const fn from_raw_unchecked(raw: u32) -> ElementId
pub const fn from_raw_unchecked(raw: u32) -> ElementId
Construct from the underlying type without any checks.
Sourcepub const fn from_usize(value: usize) -> ElementId
pub const fn from_usize(value: usize) -> ElementId
Construct this index type from a usize.